
/*
 * filetype.c
 *
 * Copyright (C) 1993 Alain Knaff
 */
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/stat.h>
#include <string.h>

#include "zlibc.h"
#include <syscall.h>
#undef HAVE_PROC
static int is_initialised=0;
static FilenameActions *filenameActions;
static char *default_uncompressor[]= { UNCOMPR, 0 };

#ifndef SECURITY
static char *custom_uncompressor[]= { 0, 0 };
#endif

char *zlib_ext;
int zlib_extlen;

int zlib_mode;
char *zlib_tmp;

char **zlib_uncompressor;

#define ENV_PREFIX "ZLIB_"

char *mode_table[] = {
  "DISABLE",
  "READDIR_COMPR",
  "VERBOSE",
  "UNLINK",
  "NOCONF"};

#ifdef NEED_PROTOTYPES
void zlib_initialise(void)
#else
void zlib_initialise()
#endif
{
  int modemask;

#ifndef SECURITY
  int i;
  char env_buffer[1024];
  char *value;
#endif

  if ( !is_initialised ){
    CommandActions *cma;

#ifdef HAVE_PROC
    char cmdline[1025];
    int fd,n;
#else
    char *cmdline;
#endif
    char *progname=0;
    

#ifdef HAVE_PROC
    /* get command line */    
    strcpy(cmdline, "unknown");
    fd=syscall(SYS_open,"/proc/self/cmdline", O_RDONLY);
    if ( fd > 0 ){
      cmdline[1024]='\0';
      n = read( fd, cmdline, 1024 );
      if ( n < 1 )
	cmdline[0] = '\0';
      cmdline[n] = '\0';
      close(fd);
    } 
#else
    /* where does the command line live on a sun ? */
    /* horrible undocumented kludge... This might even work on a proc-less 
     * linux*/
    do {
      extern char **environ;
      int zerosfound;

      if ( environ == 0 ){
	cmdline="";
	break;
      }
      cmdline = *environ;

      if ( cmdline == 0 ){
	cmdline="";
	break;
      }

      zerosfound = 0;
      while(zerosfound < 2 ){
	cmdline--;
	if ( *cmdline == 0 )
	  zerosfound++;
	else
	  zerosfound = 0;
      }
      cmdline +=2;
    } while(0);
#endif

    progname = strrchr(cmdline, '/' );

    if ( !progname )
      progname = cmdline;
    else
      progname++;

    zlib_mode = modemask = 0;

#ifdef SECURITY
    zlib_ext = EXT;
    zlib_extlen = strlen(zlib_ext);
    zlib_uncompressor = default_uncompressor;
    zlib_tmp = TMP;
#else
    zlib_tmp = getenv("ZLIB_TMP");
    if ( zlib_tmp == 0 )
      zlib_tmp = TMP;

    zlib_ext= getenv("ZLIB_EXT");
    if ( zlib_ext == 0 )
      zlib_ext = EXT;
    zlib_extlen = strlen(zlib_ext);
    if ( zlib_extlen > MAXEXTLEN ){
      fprintf(stderr,"extension too long, taking default\n");
      zlib_ext = EXT;
    }

    custom_uncompressor[0] = getenv("ZLIB_UNCOMPRESSOR");
    if ( custom_uncompressor[0] ){
      zlib_uncompressor = custom_uncompressor;
    } else
      zlib_uncompressor = default_uncompressor;

    strcpy(env_buffer, ENV_PREFIX);
    for ( i=0; i< ( sizeof(mode_table) / sizeof(char *) ); i++){
      strcpy( env_buffer+(sizeof( ENV_PREFIX ) - 1 ), mode_table[i] );
      if ( (value=getenv ( env_buffer )) ){
	if ( ( strcmp( value, "on" ) == 0 ) || ( strcmp( value, "1" ) == 0 )){
	  zlib_mode |= ( 1 << ( i + 1 ));
	}else 
	  if ( ( strcmp( value, "off" ) ) && ( strcmp( value, "0" ) ))
	    continue;
	modemask |= ( 1 << ( i + 1 ));	  
      }
    }
#endif

    if ( zlib_mode & CM_DISAB){
      is_initialised = 2;
      return;
    }

    if ( zlib_mode & CM_VERBOSE)
      fprintf(stderr,"progname = %s\n",progname);

    is_initialised = 1;

    /* get user configuration */
#ifndef NO_RT_CONF
    if ( ! ( zlib_mode & CM_NORTCONF ) )
      zlib_getuserconf(progname, &filenameActions, &zlib_mode, &modemask);
#endif

    if ( modemask != CM_ALL_MODES ){
      /* if we couldn't get the user configuration, use compiled in defaults */
      cma = zlib_commandActions;
      while ( cma->name && strcmp(cma->name, progname ) )
	cma++;

      if ( ( modemask & CM_HAVE_FA ) == 0 )
	filenameActions = cma->actions;

      zlib_mode |= ( cma->cm_type & ~modemask);
    }

    is_initialised = 2;
    /* initialisation completely done */
  } 
}


#ifdef NEED_PROTOTYPES
int zlib_getfiletype(__const char *name)
#else
int zlib_getfiletype(name)
char *name;
#endif
{
#ifdef NEED_PROTOTYPES
  __const 
#endif
  char *basename;

  FilenameActions *fa;
  int length;
  int dirlength;
  int basenamelength;

  /* preprocessing */
  basename = strrchr( name, '/' );
  if ( !basename )
    basename = name;
  else
    basename++;
  
  dirlength = basename - name;
  basenamelength = strlen(basename);
  length = dirlength + basenamelength;

  zlib_initialise();

  if ( is_initialised == 1 )
    /* transition period, loading user supplied defaults */
    return PM_SHOW_PIPE;

  fa = filenameActions;
  while ( 1 ){
    switch( fa->fa_type ){
      /* before calling strncmp, the length are compared in order to speed up
       * the search. In most cases, the length doesn't match, this way we don't
       * need a costly call to strncmp */
    case FA_ALL:
      return fa->pipe_mode;
    case FA_DIR:
      if (fa->namelength == dirlength &&
	  strncmp( fa->name, name, dirlength ) == 0 )
	return fa->pipe_mode;
      break;
    case FA_SUBDIR:
      if (fa->namelength <= dirlength && 
	  strncmp( fa->name, name, fa->namelength ) == 0 )
	return fa->pipe_mode;
      break;
    case FA_BASENAME:
      if ( fa->namelength == basenamelength &&
	  strcmp( fa->name, basename ) == 0 )
	return fa->pipe_mode;
      break;
    case FA_FILENAME:
      if ( fa->namelength == length &&
	  strcmp( fa->name, name ) == 0 )
	return fa->pipe_mode;
      break;
    case FA_SUFFIX:
      if ( fa->namelength <= basenamelength &&
	  strcmp(fa->name, basename + (basenamelength - fa->namelength)) == 0)
	return fa->pipe_mode;
      break;
    default:
      fprintf(stderr,"Error in filenameActions %x\n", fa->fa_type);
      return 0;
      break;
    }
    fa++;
  }
}


