/*
 * zlibc.h
 *
 * Copyright (C) 1993 Alain Knaff
 */
#include "config.h"


#define FA_ALL	    0

#define FA_DIR      1
#define FA_SUBDIR   2
/* FA_DIR and FA_SUBDIR should be specified with their trailing slash */

#define FA_BASENAME 3
#define FA_SUFFIX   4
#define FA_FILENAME 5

#define PM_USE_TMP_FILE 1
#define PM_HIDE_PIPE	2
#define PM_SHOW_PIPE    3     
#define PM_LEAVE_COMPR 	4

typedef struct FilenameActions {
  int fa_type;
  char *name;
  int namelength;
  int pipe_mode;
} FilenameActions;

/* the file actions have been read */
#define CM_HAVE_FA 0x1

/* disable package altogether */
#define CM_DISAB 0x2

/* readdir lists compressed files */
#define CM_READDIR_COMPR 0x4

/* print comments for each intercepted syscall */
#define CM_VERBOSE 0x8

/* unlinks compressed files when user programs asks to unlink 
 * uncompressed file */
#define CM_UNLINK 0x10

/* doesn't read run time configuration file */
#define CM_NORTCONF 0x20

/* all available file-settable modes or'ed together */
#define CM_ALL_MODES 0x1d

/* none of these flags */
#define CM_NONE 0

typedef struct CommandActions {
  char *name;
  int cm_type;  
  FilenameActions *actions;
} CommandActions;


extern CommandActions zlib_commandActions[];

#define FREC(a,b,c) { a, b, sizeof(b) - 1, c }


#ifdef linux

#define access __access
#define chmod __chmod
#define chown __chown
#define lstat __lstat
#define open __open
#define readlink __readlink
#define unlink __unlink
#define link __link
#define symlink __symlink

#define NEED_PROTOTYPES

#define HAVE_STRSEP

#define HAVE_PROC

#endif


#ifdef NEED_PROTOTYPES
void zlib_initialise(void);
#else
void zlib_initialise();
#endif

#ifdef NEED_PROTOTYPES
int zlib_getfiletype(__const char *name);
#else
int zlib_getfiletype();
#endif


#ifdef NEED_PROTOTYPES
void zlib_getuserconf(char *progname, 
		      FilenameActions **filenameActions, 
		      int *mode, int *modemask);
#else
void zlib_getuserconf();

#endif

extern char *zlib_ext;
extern int zlib_extlen;
extern int zlib_mode;
extern char *zlib_tmp;
extern char **zlib_uncompressor;



/* default configuration */
#ifndef TMP
#define TMP "/tmp/zlibc"
#endif

#ifndef UNCOMPR
#define UNCOMPR "/usr/bin/gzip", "-dc"
#endif

#ifndef EXT
#define EXT ".gz"
#endif

#ifndef MAXEXTLEN
#define MAXEXTLEN 5
#endif

#ifndef  HOMECONF
#ifndef NOHOMECONF
#define HOMECONF ".zlibrc"
#endif
#endif

#ifndef SYSTEMCONF
#ifndef NOSYSTEMCONF
#define SYSTEMCONF "/usr/lib/zlibrc"
#endif
#endif


